@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:	10.10.2003
-// RCS-ID:      $Id: MainWin.h,v 1.26 2008/10/25 21:06:29 ntalex Exp $
+// RCS-ID:      $Id: MainWin.h,v 1.34 2009/06/11 23:19:21 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,11 +13,14 @@
 
 #include <wx/wx.h>
 #include <wx/image.h>
+#include <wx/docview.h>
+#include <wx/html/helpctrl.h>
 #include "DirCtrl.h"
 #include <wxVillaLib/Thumbnails.h>
 #include "MenuEditor.h"
 #include "TitlesetManager.h"
-
+#include "StatusBar.h"
+#include "Cache.h"
 
 // begin wxGlade: dependencies
 #include <wx/splitter.h>
@@ -29,10 +32,11 @@ public:
     // begin wxGlade: MainWin::ids
     enum {
         MENU_BURN_ID = 2105,
+        MENU_DVD_ADD_FILE_ID = 2110,
         MENU_DVD_ADD_MENU_ID = 2111,
         MENU_DVD_ADD_VMMENU_ID = 2112,
         MENU_DVD_ADD_TITLESET_ID = 2113,
-        MENU_DVD_OPTIONS_ID = 2110,
+        MENU_DVD_OPTIONS_ID = 2119,
         MENU_SETTINGS_ID = 2120,
         DIR_TREE_ID = 2000,
         THUMBNAILS_ID = 2001,
@@ -41,18 +45,32 @@ public:
         SPLIITTER_TITLE_ID = 2010
     };
     // end wxGlade
-
-    MainWin();
-    /** loads a project file */
-	void Open(wxString filename);
-    /** adds a title to dvd */
-	void AddTitle(wxString filename) { m_titlesetManager->AddVideo(filename); }
-    void Burn(bool start_close);
     
+    /**
+     * Constructor
+     */
+    MainWin();
+    /**
+	 * Desstructor
+	 */
+	~MainWin();
+	/** Creates new project */
+	void NewDVD(wxString discLabel, DiscCapacity capacity, int videoBitrate,
+			VideoFormat videoFormat, AudioFormat audioFormat, AspectRatio aspectRatio);
+    /** Loads a project file */
+	void Open(wxString filename);
+	/** Shows open dialog */
+	void ShowOpenDlg();
+    /** Adds a title to dvd */
+	void AddTitle(wxString filename) { m_titlesetManager->AddVideo(filename); }
+	/** Starts a burn process */
+    void Burn(bool start_close);
+    /** Returns the file history */
+    wxFileHistory& GetFileHistory() { return m_fileHistory; }
+	
   private:
     // begin wxGlade: MainWin::attributes
     wxMenuBar* m_menubar;
-    wxStatusBar* m_statusbar;
     wxToolBar* m_dirBar;
     DirCtrl* m_dirTree;
     wxStaticText* m_thumbLabel;
@@ -64,6 +82,8 @@ public:
     TitlesetManager* m_titlesetManager;
     wxSplitterWindow* m_splitterTitle;
     // end wxGlade
+    StatusBar* m_statusbar;
+    wxHtmlHelpController m_helpController;
 	
     // begin wxGlade: MainWin::methods
     void set_properties();
@@ -71,25 +91,26 @@ public:
     // end wxGlade
 	
 	DVD* m_dvd;
-    wxString m_lastDir;
+	wxString m_lastDir;
 	wxString m_filename; /** project file name */
-    bool m_changed;
-    
-    /** creates new project */
-    void NewDVD(wxString volumeName, VideoFormat videoFormat, AudioFormat audioFormat);
-    /** must be called after m_dvd was changed */
+	bool m_changed;
+	wxFileHistory m_fileHistory;
+	
+	Cache m_cache;
+	
+    /** It must be called after m_dvd was changed */
 	void UpdateDVD();
-    
+	
     /** sets project status to changed */
     void SetChanged(bool changed);
     /** returns true if project is changed */
 	bool GetChanged() { return m_changed; }
     void OnChangedMenu(wxCommandEvent& event);
 	void OnChangedDVD(wxCommandEvent& event);
-    
+	
     /** checks if project is saved and asks for saving */
     bool TestSaved();
-    
+	
 	wxToolBar* m_toolbar;
 	int m_splitterDirSash;
 	int m_splitterTitleSash;
@@ -111,24 +132,26 @@ public:
 	void OnUpdateUISetBackground(wxUpdateUIEvent& event);
 	void OnRefresh(wxCommandEvent& event);
 	
-	void OnNew(wxCommandEvent& event);
-	void OnOpen(wxCommandEvent& event);
-	void OnSave(wxCommandEvent& event);
-	void OnSaveAs(wxCommandEvent& event);
-	void OnBurn(wxCommandEvent& event);
-	void OnExit(wxCommandEvent& event);
-	void OnAddMenu(wxCommandEvent& WXUNUSED(event));
-	void OnAddVmMenu(wxCommandEvent& WXUNUSED(event));
-	void OnAddTitleset(wxCommandEvent& WXUNUSED(event));
-	void OnAddTitlesetUpdateUI(wxUpdateUIEvent& event);
-	void OnDVDOptions(wxCommandEvent& event);
-	void OnSettings(wxCommandEvent& event);
-	void OnAbout(wxCommandEvent& event);
+	virtual void OnNew(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnOpen(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnSave(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnSaveAs(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnBurn(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnExit(wxCommandEvent& event); // wxGlade: <event_handler>
+    virtual void OnAddFile(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnAddMenu(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnAddVmMenu(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnAddTitleset(wxCommandEvent &event); // wxGlade: <event_handler>
+	virtual void OnAddTitlesetUpdateUI(wxUpdateUIEvent& event); // wxGlade: <event_handler>
+	virtual void OnDVDOptions(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnSettings(wxCommandEvent& event); // wxGlade: <event_handler>
+	virtual void OnAbout(wxCommandEvent& event); // wxGlade: <event_handler>
+    virtual void OnHelpContents(wxCommandEvent &event); // wxGlade: <event_handler>
 	
 	void OnSplitterTitleSash(wxSplitterEvent &event);
 	void OnResize(wxSizeEvent &event);
 	void OnClose(wxCloseEvent &event);
     DECLARE_EVENT_TABLE()
-};
+}; // wxGlade: end class
 
 #endif // MAINWIN_H
