@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu Object
 // Author:      Alex Thuering
 // Created:	04.11.2006
-// RCS-ID:      $Id: MenuObject.cpp,v 1.7 2008/03/02 18:56:21 ntalex Exp $
+// RCS-ID:      $Id: MenuObject.cpp,v 1.11 2009/08/03 21:59:50 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -21,30 +21,41 @@
 
 ///////////////////////////// MenuAction /////////////////////////////////////
 
-wxString MenuAction::GetAction()
-{
-  wxString action = m_action;
-  if (!action.length())
-  {
-	int tsi = GetActionTsi();
-	bool menu = GetActionPgci()%2 == 0;
-	int pgci = GetActionPgci()/2;
-	action = _T("jump ");
-	if (tsi == -1)
-	  action += _T("vmgm ");
-	else if (tsi >= 0)
-	  action += wxString::Format(_T("titleset %d "), tsi+1);
-	if (menu)
-	  action += _T("menu");
-	else
-	  action += _T("title");
-	if (!menu || pgci>0 || tsi == -2)
-	  action += wxString::Format(_T(" %d"), pgci+1);
-	if (GetActionChapter() > 0)
-	  action += wxString::Format(_T(" chapter %d"), GetActionChapter()+1);
-	action += _T(";");
-  }
-  return action;
+wxString MenuAction::GetId(bool translate) {
+	return m_id;
+}
+
+wxString MenuAction::GetAction(bool translate) {
+	wxString action = m_action;
+	if (!action.length()) {
+		int tsi = GetActionTsi();
+		bool menu = GetActionPgci() % 2 == 0;
+		int pgci = GetActionPgci() / 2;
+		action = translate ? _("jump to") : wxT("jump");
+		action += wxT(" ");
+		if (tsi >= 0) {
+			action += translate ? _("titleset") : wxT("titleset");
+			action += wxString::Format(_T(" %d "), tsi + 1);
+		}
+		if (menu) {
+			if (tsi == -1)
+				action += translate ? _("vmMenu") : wxT("vmgm menu");
+			else
+				action += translate ? _("menu") : wxT("menu");
+			action += wxT(" ");
+		} else
+			action += translate ? _("title") : _T("title");
+		if (!menu || pgci > 0 || tsi == -2)
+			action += wxString::Format(_T(" %d"), pgci + 1);
+		if (GetActionChapter() > 0) {
+			action += wxT(" ");
+			action += translate ? _("chapter") : _T("chapter");
+			action += wxString::Format(_T(" %d"), GetActionChapter() + 1);
+		}
+		if (!translate)
+			action += wxT(";");
+	}
+	return action;
 }
 
 wxSvgXmlNode* MenuAction::GetXML(DVDFileType type, bool button) {
@@ -352,6 +363,14 @@ void MenuObject::AddUse(wxString id, int x, int y, int width, int height)
   	m_svg->GetElementById(wxT("objects"))->AppendChild(m_use);
 }
 
+wxString MenuObject::GetId(bool translate) {
+	if (!translate)
+		return m_id;
+	long l = 0;
+	m_id.Mid(IsButton() ? 6 : 3).ToLong(&l);
+	return (IsButton() ? _("button") : _("object")) + wxString::Format(wxT(" %d"), l);
+}
+
 int MenuObject::GetX() { return m_use->GetX().GetBaseVal(); }
 void MenuObject::SetX(int value) { m_use->SetX(value); }
 
@@ -836,14 +855,27 @@ bool MenuObject::PutXML(wxSvgXmlNode* node)
     return false;
   
   wxFileName fn(fileName);
+  wxString dir = IsButton() ? BUTTONS_DIR : OBJECTS_DIR;
   if (fn.IsRelative()) {
-    wxString dir = IsButton() ? BUTTONS_DIR : OBJECTS_DIR;
 	if (wxFileName::FileExists(dir + fileName))
 	  fileName = dir + fileName;
     else if (wxFileName::FileExists(dir + wxT("deprecated") + wxFILE_SEP_PATH + fileName))
 	  fileName = dir + wxT("deprecated") + wxFILE_SEP_PATH + fileName;
-    else
+    else if (wxFileName::FileExists(dir + fn.GetFullName()))
+	  fileName = dir + fn.GetFullName();
+    else {
+      wxLogError(_("can't open file '%s'"), fileName.c_str());
       return false;
+    }
+  } else if (!wxFileExists(fileName)) {
+	  if (wxFileName::FileExists(dir + fn.GetFullName()))
+	  	  fileName = dir + fn.GetFullName();
+	  else if (wxFileName::FileExists(dir + wxT("deprecated") + wxFILE_SEP_PATH + fn.GetFullName()))
+	  	  fileName = dir + wxT("deprecated") + wxFILE_SEP_PATH + fn.GetFullName();
+	  else {
+		  wxLogError(_("can't open file '%s'"), fileName.c_str());
+		  return false;
+	  }
   }
   
   node->GetPropVal(_T("id"), &m_id);
