@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.h,v 1.6 2008/05/18 19:17:02 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.h,v 1.10 2009/07/22 06:37:35 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,11 +14,11 @@
 #include <wx/wx.h>
 #include <wxVillaLib/PropDlg.h>
 #include "DVD.h"
+#include "VobListBox.h"
 
 class TitlePropDlg: public wxPropDlg {
 public:
-	TitlePropDlg(wxWindow *parent, DVD* dvd, Audio* audio, Video* video,
-			Pgc* pgc, Vob* vob);
+	TitlePropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci, int vobi);
 	virtual ~TitlePropDlg() {}
 	
 private:
@@ -27,12 +27,20 @@ private:
 	Video* m_video;
 	Pgc* m_pgc;
 	Vob* m_vob;
-	int m_formatCtrlIdx;
+	int m_vobi;
+	VobListBox* m_vobListBox;
+	wxCheckBox* m_doNotTranscodeCheckBox;
+	wxButton* m_addBt;
+	wxButton* m_delBt;
 	void CreatePropPanel(wxSizer* sizer);
 	bool SetValues();
-	void CreateButtonPane(wxSizer* sizer, bool resetButton = false);
+	void CreateButtonPane(wxSizer* sizer, bool resetButton = false, bool dontShowCheckbox = false);
+	void OnSelectItem(wxCommandEvent& event);
+	void OnAddBt(wxCommandEvent& event);
+	void OnRemoveBt(wxCommandEvent& event);
 	void OnAV(wxCommandEvent& event);
 	void OnDoNotTranscodeCheck(wxCommandEvent& event);
+	void UpdateDoNotTranscodeCheck();
 	DECLARE_EVENT_TABLE()
 };
 
