@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.20 2008/11/09 19:59:55 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.30 2009/06/14 16:55:49 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,6 +14,7 @@
 #include "MPEG.h"
 #include "Config.h"
 #include "Utils.h"
+#include "MessageDlg.h"
 #include <wxVillaLib/ThumbnailFactory.h>
 #include <wxVillaLib/ImageProc.h>
 #include <wxVillaLib/utils.h>
@@ -80,13 +81,19 @@ public:
 		} else if (wxThumbnails::IsAudio(filename)) {
 			m_pOwner->AddAudio(filename);
 			return true;
-		} else if (ext == wxT("sub") || ext == wxT("srt") || ext == wxT("ssa") || ext == wxT("smi") || ext == wxT("rt") || ext == wxT("txt") || ext == wxT("aqt") || ext == wxT("jss") || ext == wxT("js") || ext == wxT("ass")) {
+		} else if (wxThumbnails::IsSubtitle(filename)) {
 			m_pOwner->AddSubtitles(filename);
 			return true;
 		}
-
-		wxLogError(filename + _(": unknown format"));
-		return false;
+		
+		// try to load it as video
+		wxFfmpegMediaDecoder decoder;
+		if (!decoder.Load(filename)) {
+			wxLogError(filename + _(": unknown format"));
+			return false;
+		}
+		m_pOwner->AddVideo(filename, defDrag == wxDragMove);
+		return true;
 	}
 protected:
 	TitlesetManager *m_pOwner;
@@ -105,6 +112,7 @@ enum {
 	MENU_TITLE_ADD_ID,
 	MENU_TITLE_MOVE_ID,
 	MENU_MAIN_ADD_ID,
+	MENU_ADD_FILE_ID,
 	MENU_ADD_MENU_ID,
 	MENU_ADD_VMMENU_ID,
 	MENU_ADD_TITLESET_ID,
@@ -113,6 +121,7 @@ enum {
 };
 
 BEGIN_EVENT_TABLE(TitlesetManager, wxThumbnails)
+	EVT_MENU(MENU_ADD_FILE_ID, TitlesetManager::OnAddFile)
 	EVT_MENU(MENU_ADD_MENU_ID, TitlesetManager::OnAddMenu)
 	EVT_MENU(MENU_ADD_VMMENU_ID, TitlesetManager::OnAddVmMenu)
 	EVT_MENU(MENU_ADD_TITLESET_ID, TitlesetManager::OnAddTitleset)
@@ -141,6 +150,7 @@ TitlesetManager::TitlesetManager(wxWindow* parent, int id) :
 	m_selectedMenu = -1;
 
 	wxMenu* addMenu = new wxMenu;
+	addMenu->Append(MENU_ADD_FILE_ID, _("&File..."));
 	addMenu->Append(MENU_ADD_MENU_ID, _("&Menu"));
 	addMenu->Append(MENU_ADD_VMMENU_ID, _("&vmMenu"));
 	addMenu->Append(MENU_ADD_TITLESET_ID, _("&Titleset"));
@@ -161,7 +171,9 @@ TitlesetManager::TitlesetManager(wxWindow* parent, int id) :
 	item = new wxMenuItem(m_titleMenu, MENU_TITLE_COPY_ID, _("&Copy"));
 	item->SetBitmap(wxArtProvider::GetBitmap(wxART_COPY, wxART_MENU));
 	m_titleMenu->Append(item);
-	m_titleMenu->Append(MENU_TITLE_DEL_ID, _("&Delete"));
+	item = new wxMenuItem(m_titleMenu, MENU_TITLE_DEL_ID, _("&Delete"));
+	item->SetBitmap(wxArtProvider::GetBitmap(wxART_DELETE, wxART_MENU));
+	m_titleMenu->Append(item);
 	m_titleMenu->Append(MENU_TITLE_PROP_ID, _("&Properties..."));
 
 	SetPopupMenu(m_titleMenu);
@@ -169,6 +181,7 @@ TitlesetManager::TitlesetManager(wxWindow* parent, int id) :
 }
 
 TitlesetManager::~TitlesetManager() {
+	// nothing to do
 }
 
 void TitlesetManager::SetDVD(DVD* dvd) {
@@ -255,7 +268,7 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 				imgRect.y + img.GetHeight() - noaudioBitmap.GetHeight());
 	} else if (vob->GetAudioFilenames().GetCount() > 0) {
 		wxString text = vob->GetAudioFilenames()[0].AfterLast(wxT('.')).Upper();
-		for (int fi = 1; fi < vob->GetAudioFilenames().GetCount(); fi++)
+		for (int fi = 1; fi < (int)vob->GetAudioFilenames().GetCount(); fi++)
 			text += wxT(",") + vob->GetAudioFilenames()[fi].AfterLast(wxT('.')).Upper();
 		dc.SetTextForeground(*wxWHITE);
 		wxFont font(9, wxFONTFAMILY_ROMAN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
@@ -379,11 +392,12 @@ int TitlesetManager::GetIndex(int id) {
 	return -1;
 }
 
-bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle) {
+bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi) {
 	if (m_dvd->GetTitlesets().Count() == 0)
 		m_dvd->GetTitlesets().Add(new Titleset);
-
-	PgcArray& pgcs = m_dvd->GetTitlesets().Last()->GetTitles();
+	
+	Titleset* titleset = tsi >= 0 ? m_dvd->GetTitlesets()[tsi] : m_dvd->GetTitlesets().Last();
+	PgcArray& pgcs = titleset->GetTitles();
 	Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
 	Vob* vob = pgc && pgc->GetVobs().Count()>0 ? pgc->GetVobs().Last() : NULL;
 	if (vob && vob->GetFilename().length() && !vob->HasAudio()) {
@@ -435,6 +449,8 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle) {
 			cStr += wxString::Format(wxT("%02d:00"), t%60);
 		t += s_config.GetDefChapterLength();
 	}
+	if (!createTitle && pgc != NULL)
+		cStr = wxT("0:00") + wxString(cStr.length() ? wxT(",") + cStr : wxT(""));
 	vob->SetChapters(cStr);
 	if (createTitle || pgc == NULL) {
 		pgc = new Pgc;
@@ -505,7 +521,7 @@ void TitlesetManager::AddImage(const wxString& fname, bool createTitle) {
 
 	Slideshow* slideshow = pgc->GetSlideshow();
 	if (slideshow == NULL) {
-		slideshow = new Slideshow;
+		slideshow = new Slideshow(m_dvd->GetVideoFormat(), m_dvd->GetAspectRatio());
 		Vob* vob = new Vob(slideshow);
 		pgc->GetVobs().Add(vob);
 	}
@@ -582,7 +598,7 @@ void TitlesetManager::AddMenu() {
 	int tsi = m_dvd->GetTitlesets().Count()-1;
 	if (GetSelected()>=0)
 		tsi = DVD::GetTsi(GetSelectedId());
-	int pgci = m_dvd->AddMenu(m_dvd->GetVideoFormat(), tsi);
+	int pgci = m_dvd->AddMenu(tsi);
 	UpdateItems();
 	SelectMenu(tsi, pgci);
 	SendDvdChangedEvent();
@@ -590,7 +606,7 @@ void TitlesetManager::AddMenu() {
 
 void TitlesetManager::AddVmMenu() {
 	int tsi = -1;
-	int pgci = m_dvd->AddMenu(m_dvd->GetVideoFormat(), tsi);
+	int pgci = m_dvd->AddMenu(tsi);
 	UpdateItems();
 	SelectMenu(tsi, pgci);
 	SendDvdChangedEvent();
@@ -601,6 +617,29 @@ void TitlesetManager::AddTitleset() {
 	AddMenu();
 }
 
+void TitlesetManager::OnAddFile(wxCommandEvent& event) {
+	wxString video = wxThumbnails::GetVideoExtWildcard();
+	wxString audio = wxThumbnails::GetAudioExtWildcard();
+	wxString subtitle = wxThumbnails::GetSubtitleExtWildcard();
+	wxFileDialog dialog(this, _("Choose a file"), wxT(""), wxT(""),
+			_("Audio/Video/Subtitle Files") + wxString::Format(wxT("|%s%s%s|"), video.c_str(), audio.c_str(), subtitle.c_str())
+			+ _("Audio Files") + wxString::Format(wxT(" (%s)|%s|"), audio.c_str(), audio.c_str())
+			+ _("Video Files") + wxString::Format(wxT(" (%s)|%s|"), video.c_str(), video.c_str())
+			+ _("Subtitle Files") + wxString::Format(wxT(" (%s)|%s|"), subtitle.c_str(), subtitle.c_str()),
+			wxFD_OPEN);
+	// set file browser dir
+	dialog.SetDirectory(s_config.GetLastAddDir() + wxFILE_SEP_PATH);
+	if (dialog.ShowModal() == wxID_OK) {
+		if (wxThumbnails::IsVideo(dialog.GetPath()))
+			AddVideo(dialog.GetPath());
+		else if (wxThumbnails::IsAudio(dialog.GetPath()))
+			AddAudio(dialog.GetPath());
+		else
+			AddSubtitles(dialog.GetPath());
+		s_config.SetLastAddDir(dialog.GetDirectory());
+	}
+}
+
 void TitlesetManager::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event) {
 	int cnt = m_dvd->GetTitlesets().Count();
 	event.Enable(cnt>0 && m_dvd->GetTitlesets()[cnt-1]->GetTitles().Count());
@@ -671,14 +710,18 @@ void TitlesetManager::OnDelete(wxCommandEvent& WXUNUSED(event)) {
 	int pgci = DVD::GetPgci(id);
 	bool isMenu = DVD::IsMenu(id);
 	int vobi = DVD::GetVobi(id);
-	wxString caption = isMenu ? _("Menu") : _T("Title");
-	caption += wxString::Format(wxT(" %d"), pgci+1);
-	if (vobi>0)
-		caption += wxString::Format(_T("-%d"), vobi+1);
-	if (wxMessageBox(wxString::Format(_("Do you really want to delete %s?"), caption.c_str()),
-			_("Titleset Manager"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
-		return;
-  if (isMenu) {
+	if (s_config.GetShowTitleDeletePrompt()) {
+		wxString caption = isMenu ? _("Menu") : _T("Title");
+		caption += wxString::Format(wxT(" %d"), pgci+1);
+		if (vobi>0)
+			caption += wxString::Format(_T("-%d"), vobi+1);
+		MessageDlg msgDlg(this, wxString::Format(_("Do you really want to delete %s?"), caption.c_str()),
+				_("Titleset Manager"), wxYES_NO|wxICON_QUESTION);
+		if (msgDlg.ShowModal() != wxYES)
+			return;
+		s_config.SetShowTitleDeletePrompt(!msgDlg.DontShowAgain());
+	}
+	if (isMenu) {
 		Titleset* ts= NULL;
 		if (tsi>=0)
 			ts = m_dvd->GetTitlesets()[tsi];
@@ -761,21 +804,19 @@ void TitlesetManager::OnProps(wxCommandEvent& WXUNUSED(event)) {
 	int pgci = DVD::GetPgci(id);
 	bool isMenu = DVD::IsMenu(id);
 	int vobi = DVD::GetVobi(id);
-	if (isMenu) {
+	if (isMenu) { // menu
 		MenuPropDlg propDlg(this->GetParent(), m_dvd, tsi, pgci);
 		if (propDlg.ShowModal() == wxID_OK) {
 			SendMenuSelectedEvent(); // updated MenuEditor
 			SendDvdChangedEvent();
 		}
-	} else //title
-	{
-		PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
-		Pgc* pgc = pgcs[pgci];
-		TitlePropDlg propDlg(this->GetParent(), m_dvd, &pgcs.GetAudio(),
-				&pgcs.GetVideo(), pgc, pgc->GetVobs()[vobi]);
+	} else { //title
+		TitlePropDlg propDlg(this->GetParent(), m_dvd, tsi, pgci, vobi);
 		if (propDlg.ShowModal() == wxID_OK)
 			SendDvdChangedEvent();
 	}
+	m_pointed = -1;
+	Refresh();
 }
 
 void TitlesetManager::OnCopy(wxCommandEvent& WXUNUSED(event)) {
@@ -793,7 +834,7 @@ void TitlesetManager::OnPaste(wxCommandEvent& WXUNUSED(event)) {
 	int tsi = m_dvd->GetTitlesets().Count()-1;
 	if (GetSelected()>=0)
 		tsi = DVD::GetTsi(GetSelectedId());
-	Menu* menu = new Menu(m_dvd->GetVideoFormat());
+	Menu* menu = new Menu(m_dvd->GetVideoFormat(), m_dvd->GetAspectRatio());
 	wxSvgXmlDocument doc;
 	if (!GetXmlFromClipboard(DATAFORMAT_MENU, doc))
 		return;
