@@ -3,11 +3,11 @@
 // Purpose:     Properties dialog (base class for all properties dialogs)
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.cpp,v 1.8 2008/03/02 19:02:52 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.cpp,v 1.16 2009/06/07 20:04:00 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
- 
+
 #include "PropDlg.h"
 #include <wx/fontdlg.h>
 #include <wx/colordlg.h>
@@ -15,65 +15,60 @@
 #include <wx/dirdlg.h>
 #include <wx/grid.h>
 #include <wx/spinctrl.h>
+#include <wx/statline.h>
 
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////// ColourPanel ////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-class ColourPanel: public wxPanel
-{
-  public:
-	ColourPanel(wxWindow* parent, wxWindowID id): wxPanel(parent, id,
-	  wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL)
-	{
-	  m_bgColour = GetBackgroundColour();
+class ColourPanel: public wxPanel {
+public:
+	ColourPanel(wxWindow* parent, wxWindowID id) :
+		wxPanel(parent, id, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL) {
+		m_bgColour = GetBackgroundColour();
 	}
-	
-	inline void SetColour(wxColour colour)
-	{
-	  m_colour = colour;
-	  if (colour.Ok())
-		SetBackgroundColour(colour);
-	  else
-		SetBackgroundColour(m_bgColour);
-	  Refresh();
+
+	inline void SetColour(wxColour colour) {
+		m_colour = colour;
+		if (colour.Ok())
+			SetBackgroundColour(colour);
+		else
+			SetBackgroundColour(m_bgColour);
+		Refresh();
 	}
-	
-	inline wxColour GetColour()	{ return m_colour; }
-	
-  protected:
+
+	inline wxColour GetColour() {
+		return m_colour;
+	}
+
+protected:
 	wxColour m_colour;
 	wxColour m_oldColour;
 	wxColour m_bgColour;
-	
-	void OnPaint(wxPaintEvent &event)
-	{
-	  wxPaintDC dc(this);  
-	  PrepareDC(dc);
-	  if (!m_colour.Ok())
-	  {
-		dc.SetPen(*wxBLACK_PEN);
-		dc.SetBrush(*wxTRANSPARENT_BRUSH);
-		int w = GetClientSize().GetWidth();
-		int h = GetClientSize().GetHeight();
-		dc.DrawLine(0, 0, w, h);
-		dc.DrawLine(w, 0, 0, h);
-	  }
+
+	void OnPaint(wxPaintEvent &event) {
+		wxPaintDC dc(this);
+		PrepareDC(dc);
+		if (!m_colour.Ok()) {
+			dc.SetPen(*wxBLACK_PEN);
+			dc.SetBrush(*wxTRANSPARENT_BRUSH);
+			int w = GetClientSize().GetWidth();
+			int h = GetClientSize().GetHeight();
+			dc.DrawLine(0, 0, w, h);
+			dc.DrawLine(w, 0, 0, h);
+		}
 	}
-	
-	void OnClick(wxMouseEvent &event)
-	{
-	  if (m_colour.Ok())
-	  {
-		m_oldColour = GetColour();
-		SetColour(wxColour());
-	  }
-	  else
-		SetColour(m_oldColour);
-	  Refresh();
+
+	void OnClick(wxMouseEvent &event) {
+		if (m_colour.Ok()) {
+			m_oldColour = GetColour();
+			SetColour(wxColour());
+		} else
+			SetColour(m_oldColour);
+		Refresh();
 	}
-	
-  private:
+
+private:
 	DECLARE_EVENT_TABLE()
 };
 
@@ -86,9 +81,9 @@ END_EVENT_TABLE()
 ///////////////////////////// wxPropDlg //////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-enum
-{
+enum {
   RESET_BT_ID = 7830,
+  CUSTOM_CHOICE_ID,
   SELECT_FONT_BT_ID,
   SELECT_COLOR_BT_ID,
   SELECT_FILE_BT_ID,
@@ -110,10 +105,10 @@ BEGIN_EVENT_TABLE(wxPropDlg, wxDialog)
   EVT_GRID_CELL_RIGHT_CLICK(wxPropDlg::OnCellRightClick)
   EVT_GRID_CELL_CHANGE(wxPropDlg::OnCellChange)
   EVT_MENU(ROW_DELETE_ID, wxPropDlg::OnRowDelete)
+  EVT_CHOICE(CUSTOM_CHOICE_ID, wxPropDlg::OnSelectCustomChoice)
 END_EVENT_TABLE()
 
-enum PropertyType
-{
+enum PropertyType {
   propTEXT = 0,
   propSTAT_TEXT,
   propSPIN,
@@ -127,27 +122,34 @@ enum PropertyType
   propFONT
 };
 
-wxPropDlg::wxPropDlg(wxWindow* parent): wxDialog(parent, -1,
-  _("Properties"), wxDefaultPosition, wxDefaultSize,
-  wxDEFAULT_DIALOG_STYLE| wxRESIZE_BORDER)
-{
-  m_currGroupId = 0;
+wxPropDlg::wxPropDlg(wxWindow* parent, wxString title): wxDialog(parent, -1, title, wxDefaultPosition,
+		wxDefaultSize, wxDEFAULT_DIALOG_STYLE| wxRESIZE_BORDER) {
+	m_currGroupId = 0;
+	m_updateIndex = -1;
 }
 
-void wxPropDlg::Create(bool resetButton)
+void wxPropDlg::Create(bool resetButton, bool dontShowCheckbox)
 {
   propWindow = this;
   wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
   wxBoxSizer* propSizer = new wxBoxSizer(wxVERTICAL);
   CreatePropPanel(propSizer);
   sizer->Add(propSizer, 1, wxEXPAND|wxALL, 10);
-  CreateButtonPane(sizer, resetButton);
+  CreateButtonPane(sizer, resetButton, dontShowCheckbox);
   SetAutoLayout(true);
   SetSizer(sizer);
   sizer->Fit(this);
   sizer->SetSizeHints(this);
   Layout();
   Centre();
+  for (int i=0; i<(int)m_controls.GetCount(); i++) {
+	  if (m_types[i] < propCOLOUR && m_types[i] != propRADIO_GROUP
+			  && ((wxControl*) m_controls[i])->IsEnabled()
+			  && (m_types[i] != propRADIO || ((wxRadioButton*) m_controls[i])->GetValue())) {
+		  ((wxControl*) m_controls[i])->SetFocus();
+		  break;
+	  }
+  }
 }
 
 wxPropDlg::~wxPropDlg()
@@ -180,37 +182,29 @@ void wxPropDlg::OnReset(wxCommandEvent& event)
   Reset();
 }
 
-void wxPropDlg::CreatePropPanel(wxSizer* sizer)
-{
+void wxPropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton, bool dontShowCheckbox) {
+	wxBoxSizer* buttonPane = new wxBoxSizer(wxHORIZONTAL);
+	if (dontShowCheckbox)
+		AddCheckProp(buttonPane, _("&Don't show this dialog again"), false);
+	buttonPane->Add(10, 10, dontShowCheckbox ? 0 : 1, wxEXPAND);
+	wxButton* okBt = new wxButton(this, wxID_OK, _("OK"));
+	okBt->SetDefault();
+	buttonPane->Add(okBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
+	buttonPane->Add(8, 0);
+	wxButton* cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
+	buttonPane->Add(cancelBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
+	if (resetButton) {
+		buttonPane->Add(8, 0);
+		wxButton* resetBt = new wxButton(this, RESET_BT_ID, _("Reset"));
+		buttonPane->Add(resetBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
+	}
+	buttonPane->Add(10, 10, dontShowCheckbox ? 0 : 1, wxEXPAND);
+	sizer->Add(buttonPane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
 }
 
-void wxPropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton)
-{
-  wxBoxSizer* buttonPane = new wxBoxSizer(wxHORIZONTAL);
-  buttonPane->Add(10, 10, 1, wxEXPAND);
-  wxButton* okBt = new wxButton(this, wxID_OK, _("OK"));
-  okBt->SetDefault();
-#if defined(__WXGTK__) && wxCHECK_VERSION(2,5,2) && !wxCHECK_VERSION(2,5,4)
-  okBt->SetSizeHints(-1, okBt->GetSize().GetY()+8);
-#endif
-  buttonPane->Add(okBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-  buttonPane->Add(8, 0);
-  wxButton* cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
-  buttonPane->Add(cancelBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-  if (resetButton)
-  {
-    buttonPane->Add(8, 0);
-    wxButton* resetBt = new wxButton(this, RESET_BT_ID, _("Reset"));
-    buttonPane->Add(resetBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-  }
-  buttonPane->Add(10, 10, 1, wxEXPAND);
-  sizer->Add(buttonPane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
-}
-
-void wxPropDlg::Reset()
-{
-  m_updateIndex = 0;
-  CreatePropPanel(NULL);
+void wxPropDlg::Reset() {
+	m_updateIndex = 0;
+	CreatePropPanel(NULL);
 }
 
 void wxPropDlg::AddText(wxSizer* sizer, wxString text)
@@ -272,8 +266,18 @@ void wxPropDlg::AddStaticTextProp(wxSizer* sizer, const wxString& label, const w
   m_groupIds.Add(m_currGroupId);
 }
 
+
+void wxPropDlg::AddStaticLine(wxSizer* sizer, wxString caption) {
+	wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
+	wxStaticText* staticText = new wxStaticText(this, -1, caption);
+	staticText->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxBOLD, 0, _T("")));
+	sizer2->Add(staticText, 0, wxRIGHT | wxTOP | wxBOTTOM, 5);
+	sizer2->Add(new wxStaticLine(this, -1), 1, wxALIGN_CENTER_VERTICAL, 5);
+	sizer->Add(sizer2, 0, wxEXPAND, 0);
+}
+
 wxSizer* wxPropDlg::AddSpinProp(wxSizer* sizer, const wxString& label, int value,
-		int min, int max, int width, bool addSpacer) {
+		int min, int max, int width, const wxString& label2, bool addSpacer) {
 	if (!sizer) { // only update value
 		wxSpinCtrl* ctrl = (wxSpinCtrl*) m_controls[m_updateIndex];
 		ctrl->SetValue(value);
@@ -291,8 +295,12 @@ wxSizer* wxPropDlg::AddSpinProp(wxSizer* sizer, const wxString& label, int value
 	if (width > 0) {
 		pane = new wxBoxSizer(wxHORIZONTAL);
 		pane->Add(ctrl);
+		if (label2.length() > 0) {
+			pane->AddSpacer(4);
+			AddText(pane, label2);
+		}
 		if (addSpacer)
-			pane->Add(10, 10, 1, wxEXPAND);
+			pane->Add(0, 0, 1, wxEXPAND);
 		sizer->Add(pane, 1, wxEXPAND, 0);
 	} else
 		sizer->Add(ctrl, 1, wxEXPAND, 0);
@@ -366,7 +374,7 @@ void wxPropDlg::AddRadioGroupProp(wxSizer* sizer, const wxArrayString& text,
   for (int i=0; i<(int)text.GetCount(); i++) {
 	  // add checkbox control
 	  wxRadioButton* ctrl = new wxRadioButton(propWindow, -1, text[i],
-		 wxDefaultPosition, wxDefaultSize, i == 0 ? wxRB_GROUP : wxRB_SINGLE);
+		 wxDefaultPosition, wxDefaultSize, i == 0 ? wxRB_GROUP : 0);
 	  ctrl->SetValue(i == value);
 	  ctrl->Enable(!readonly);
 	  ctrl->SetName(wxString::Format(_T("%d_%d"), m_controls.GetCount(), i));
@@ -423,9 +431,8 @@ void wxPropDlg::SetComboData(wxComboBox* ctrl, const wxString& value,
   ctrl->SetValue(value);
 }
 
-wxSizer* wxPropDlg::AddChoiceProp(wxSizer* sizer, const wxString& label,
-		const wxString& value, const wxArrayString& choices,
-		int width, bool addSpacer, int id) {
+wxSizer* wxPropDlg::AddChoiceProp(wxSizer* sizer, const wxString& label, const wxString& value,
+		const wxArrayString& choices, int width, bool addSpacer, int id) {
 	if (!sizer) { // only update value
 		wxChoice* ctrl = (wxChoice*)m_controls[m_updateIndex];
 		ctrl->Clear();
@@ -441,12 +448,14 @@ wxSizer* wxPropDlg::AddChoiceProp(wxSizer* sizer, const wxString& label,
 			wxDefaultPosition, wxDefaultSize, 0, strings);
 	SetChoiceData(ctrl, value, choices);
 	wxBoxSizer* pane = NULL;
-  	if (width > 0) {
+  	if (width >= 0) {
   		pane = new wxBoxSizer(wxHORIZONTAL);
   		pane->Add(ctrl);
   		if (addSpacer)
   			pane->Add(10, 10, 1, wxEXPAND);
   		sizer->Add(pane, 1, wxEXPAND, 0);
+  		if (width > 0)
+  			ctrl->SetMinSize(wxSize(width, -1));
   	} else
   		sizer->Add(ctrl, 1, wxEXPAND, 0);
 	m_types.Add(propCHOICE);
@@ -455,17 +464,41 @@ wxSizer* wxPropDlg::AddChoiceProp(wxSizer* sizer, const wxString& label,
 	return pane;
 }
 
-void wxPropDlg::SetChoiceData(wxChoice* ctrl, const wxString& value,
-  const wxArrayString& choices)
-{
-  int sel = -1;
-  for (int i=0; i<(int)choices.GetCount(); i++)
-  {
-	ctrl->Append(choices[i]);
-	if (choices[i] == value)
-	  sel = i;
-  }
-  ctrl->SetSelection(sel);
+/**
+ * Creates label and choice control with item 'Custom'.
+ * @returns subSizer if width > 0, null otherwise
+ */
+wxSizer* wxPropDlg::AddChoiceCustomProp(wxSizer* sizer, const wxString& label, const wxString& value,
+		const wxArrayString& choices, int customItemIdx, int width, bool addSpacer) {
+	wxSizer* s = AddChoiceProp(sizer, label, value, choices, width, addSpacer, CUSTOM_CHOICE_ID);
+	GetLastControl()->SetName(wxString::Format(wxT("%d-%d"), GetLastControlIndex(), customItemIdx));
+	return s;
+}
+
+/** Sets the last control as control to enter the custom value. */
+void wxPropDlg::SetLastControlCustom(int choiceCtrlIdx, bool enable) {
+	GetLastControl()->SetName(wxString::Format(wxT("customCtrlFor%d"), choiceCtrlIdx));
+	GetLastControl()->Enable(enable);
+}
+
+void wxPropDlg::SetChoiceData(wxChoice* ctrl, const wxString& value, const wxArrayString& choices) {
+	int sel = -1;
+	for (int i = 0; i < (int) choices.GetCount(); i++) {
+		ctrl->Append(choices[i]);
+		if (choices[i] == value)
+			sel = i;
+	}
+	ctrl->SetSelection(sel);
+}
+
+void wxPropDlg::OnSelectCustomChoice(wxCommandEvent& evt) {
+	wxString name = ((wxWindow*)evt.GetEventObject())->GetName();
+	long idx = 0;
+	long itemIdx = 0;
+	if (name.BeforeLast(wxT('-')).ToLong(&idx) && name.AfterFirst(wxT('-')).ToLong(&itemIdx)) {
+		wxWindow* ctrl = this->FindWindowByName(wxString::Format(wxT("customCtrlFor%d"), idx));
+		ctrl->Enable(evt.GetSelection() == itemIdx);
+	}
 }
 
 //--- Grid property ---//
@@ -486,7 +519,7 @@ void wxPropDlg::AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data,
 
   // todo: autosize RowLableSize
   int width = 50;
-  grid->SetRowLabelSize(50);  
+  grid->SetRowLabelSize(50);
   //grid->SetRowSize(0, 60 );
   //grid->SetColSize(0, 120);
   //grid->AutoSize();
@@ -759,22 +792,18 @@ void wxPropDlg::OnSelectDir(wxCommandEvent& event)
 ////////////////////////////// Group /////////////////////////////////////////
 
 wxSizer* wxPropDlg::BeginGroup(wxSizer* sizer,
-  wxString title, wxString checkTitle, bool value, bool readonly)
+  wxString title, wxString checkTitle, bool value, bool readonly, bool vertical, int proportion)
 {
   if (!sizer) // only update value
   {
-	m_currGroupChecker = NULL;
 	if (checkTitle.length())
-	{
 	  AddCheckProp(NULL, wxEmptyString, value, readonly, GROUP_CHECK_ID);
-	  m_currGroupChecker = (wxCheckBox*)m_controls[m_updateIndex-1];
-	}
 	return NULL;
   }
   m_currGroupId++;
   wxStaticBox* sb = new wxStaticBox(propWindow, -1, title);
-  wxStaticBoxSizer* sbsizer = new wxStaticBoxSizer(sb, wxVERTICAL);
-  sizer->Add(sbsizer, 0, wxEXPAND);
+  wxStaticBoxSizer* sbsizer = new wxStaticBoxSizer(sb, vertical ? wxVERTICAL : wxHORIZONTAL);
+  sizer->Add(sbsizer, proportion, wxEXPAND);
   if (checkTitle.length())
   {
     wxFlexGridSizer* groupSizer = new wxFlexGridSizer(2, 2, 8, 4);
@@ -813,6 +842,18 @@ void wxPropDlg::OnGroupCheck(wxCommandEvent& event)
 	  ((wxWindow*) m_controls[i])->Enable(ctrl->GetValue());
 }
 
+wxControl* wxPropDlg::GetControl(int index) {
+	return (wxControl*) m_controls[index];
+}
+
+wxControl* wxPropDlg::GetLastControl() {
+	return (wxControl*) m_controls[GetLastControlIndex()];
+}
+
+int wxPropDlg::GetLastControlIndex() {
+	return m_updateIndex > 0 ? m_updateIndex - 1 : m_controls.GetCount() -1;
+}
+
 ///////////////////////// Get Values /////////////////////////////////////////
 
 wxString wxPropDlg::GetString(int index)
