@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
 // Created:		2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.12 2008/10/25 19:15:00 ntalex Exp $
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.18 2009/07/11 17:17:04 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -211,8 +211,8 @@ void wxThumbnails::ShowDir(wxString dir, int filter)
 	if (m_filter == -1 ||
         (m_filter & wxTHUMB_FILTER_VIDEOS && IsVideo(fname)) ||
 	    (m_filter & wxTHUMB_FILTER_AUDIOS && IsAudio(fname)) ||
-		(m_filter & wxTHUMB_FILTER_IMAGES &&
-		wxImage::FindHandler(fname.AfterLast('.').Lower(), -1)) ||
+	    (m_filter & wxTHUMB_FILTER_AUDIOS && IsSubtitle(fname)) ||
+		(m_filter & wxTHUMB_FILTER_IMAGES && wxImage::FindHandler(fname.AfterLast('.').Lower(), -1)) ||
         (m_filter & wxTHUMB_FILTER_CUSTOM && IsCustomFile(fname)))
 	  m_items.Add(new wxThumb(fname, caption));
     fname = wxFindNextFile();
@@ -231,35 +231,39 @@ void wxThumbnails::SetSelected(int value)
 	AddToSelection(value);
 }
 
-bool wxThumbnails::IsAudioVideo(const wxString& fname)
-{
-  return fname.AfterLast('.').Lower() == _T("mpg") ||
-	fname.AfterLast('.').Lower() == _T("mpeg") ||
-	fname.AfterLast('.').Lower() == _T("vob");
+wxString wxThumbnails::GetAudioVideoExtWildcard() {
+	return wxT("*.avi;*.ogg;*.mpg;*.mpeg;*.m2p;*.mp4;*.mkv;*.mov;*.vob;*.wmv;");
 }
 
-bool wxThumbnails::IsVideo(const wxString& fname)
-{
-  return IsAudioVideo(fname) ||
-	fname.AfterLast('.').Lower() == _T("m1v") ||
-	fname.AfterLast('.').Lower() == _T("m2v") ||
-	fname.AfterLast('.').Lower() == _T("mpv") ||
-	fname.AfterLast('.').Lower() == _T("mp4") ||
-	fname.AfterLast('.').Lower() == _T("avi") ||
-	fname.AfterLast('.').Lower() == _T("wmv") ||
-	fname.AfterLast('.').Lower() == _T("mkv");
+wxString wxThumbnails::GetVideoExtWildcard() {
+	return GetAudioVideoExtWildcard() + wxT("*.m1v;*.m2v;*.mpv;*.m4v;");
 }
 
-bool wxThumbnails::IsAudio(const wxString& fname)
-{
-  return fname.AfterLast('.').Lower() == _T("mpa") ||
-	fname.AfterLast('.').Lower() == _T("mp2") ||
-	fname.AfterLast('.').Lower() == _T("mp3") ||
-	fname.AfterLast('.').Lower() == _T("ac3") ||
-	fname.AfterLast('.').Lower() == _T("dts") ||
-	fname.AfterLast('.').Lower() == _T("pcm");
+wxString wxThumbnails::GetAudioExtWildcard() {
+	return wxT("*.mpa;*.mp2;*.mp3;*.m4a;*.ac3;*.dts;*.pcm;");
 }
 
+wxString wxThumbnails::GetSubtitleExtWildcard() {
+	return wxT("*.sub;*.srt;*.ssa;*.smi;*.rt;*.txt;*.aqt;*.jss;*.js;*.ass;");
+}
+
+bool wxThumbnails::IsAudioVideo(const wxString& fname) {
+	return GetAudioVideoExtWildcard().Find(wxT(".") + fname.AfterLast('.').Lower() + wxT(";")) != wxNOT_FOUND;
+}
+
+bool wxThumbnails::IsVideo(const wxString& fname) {
+	return GetVideoExtWildcard().Find(wxT(".") + fname.AfterLast('.').Lower() + wxT(";")) != wxNOT_FOUND;
+}
+
+bool wxThumbnails::IsAudio(const wxString& fname) {
+	return GetAudioExtWildcard().Find(wxT(".") + fname.AfterLast('.').Lower() + wxT(";")) != wxNOT_FOUND;
+}
+
+bool wxThumbnails::IsSubtitle(const wxString& fname) {
+	return GetSubtitleExtWildcard().Find(wxT(".") + fname.AfterLast('.').Lower() + wxT(";")) != wxNOT_FOUND;
+}
+
+
 void wxThumbnails::UpdateItems()
 {
   wxArrayInt selected = m_selectedArray;
